package com.sd.cainiao.service.impl;

import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.exception.ServiceException;
import com.sd.business.entity.express.po.ExpressDelivery;
import com.sd.business.entity.order.po.OrderEncasement;
import com.sd.business.entity.order.po.OrderEncasementDetail;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.service.express.ExpressDeliveryService;
import com.sd.business.service.order.OrderEncasementDetailService;
import com.sd.business.service.order.OrderEncasementService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.cainiao.entity.bo.BasicKeyBo;
import com.sd.cainiao.entity.dto.ItemParamDto;
import com.sd.cainiao.entity.dto.TakeNumParamDto;
import com.sd.cainiao.service.IsticeService;
import com.taobao.pac.sdk.cp.PacClient;
import com.taobao.pac.sdk.cp.SendSysParams;
import com.taobao.pac.sdk.cp.dataobject.request.TMS_WAYBILL_GET.*;
import com.taobao.pac.sdk.cp.dataobject.response.TMS_WAYBILL_GET.TmsWaybillGetResponse;
import com.taobao.pac.sdk.cp.dataobject.response.TMS_WAYBILL_GET.WaybillCloudPrintResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class IsticeServiceImpl implements IsticeService {

    @Autowired
    private BasicKeyBo basicKeyBo;

    @Autowired
    private OrderService orderService;

    @Autowired
    private ExpressDeliveryService expressDeliveryService;

    @Autowired
    private OrderEncasementService orderEncasementService;

    @Autowired
    private OrderEncasementDetailService orderEncasementDetailService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public synchronized void takeNum(Long orderEncasementId) {
        OrderEncasement orderEncasement = orderEncasementService.getById(orderEncasementId);
        if (orderEncasement == null || StrUtil.isNotBlank(orderEncasement.getTrackingNumber())) {
            return;
        }

        Long orderId = orderEncasement.getOrderId();
        OrderInfo orderInfo = orderService.getById(orderId);
        ExpressDelivery expressDelivery = expressDeliveryService.getById(orderInfo.getExpressDeliveryId());
        List<OrderEncasementDetail> orderEncasementDetailList = orderEncasementDetailService.list(q -> q.eq(OrderEncasementDetail::getOrderEncasementId, orderEncasementId));

        List<Long> skuSpecIdList = orderEncasementDetailList.stream().map(OrderEncasementDetail::getSkuSpecId).distinct().collect(Collectors.toList());
        Map<Long, String> skuSpecNameMap = skuSpecService.mapKV(BaseIdPo::getId, SkuSpec::getName, q -> q.in(BaseIdPo::getId, skuSpecIdList));

        TakeNumParamDto takeNumParam = new TakeNumParamDto();
        takeNumParam.setCpCode(expressDelivery.getExpressageCode());
        takeNumParam.setName(expressDelivery.getName());
        takeNumParam.setPhone(expressDelivery.getPhone());
        takeNumParam.setMobile(expressDelivery.getPhone());
        takeNumParam.setProvince(expressDelivery.getProvince());
        takeNumParam.setCity(expressDelivery.getCity());
        takeNumParam.setDistrict(expressDelivery.getDistrict());
        takeNumParam.setDetail(expressDelivery.getAddress());
        takeNumParam.setEmsTemplateUrl(expressDelivery.getTemplateUrl());

        takeNumParam.setReceiveName(orderInfo.getConsignee());
        takeNumParam.setReceiveMobile(orderInfo.getConsigneeNumber());
        takeNumParam.setReceiveProvince(orderInfo.getProvince());
        takeNumParam.setReceiveCity(orderInfo.getCity());
        takeNumParam.setReceiveDistrict(orderInfo.getCounty());
        takeNumParam.setReceiveDetail(orderInfo.getDetailedAddress());

        List<ItemParamDto> itemParamList = orderEncasementDetailList.stream().map(item -> {
            ItemParamDto itemParam = new ItemParamDto();
            itemParam.setItemName(skuSpecNameMap.get(item.getBomSpecId()));
            itemParam.setItemCount(item.getQuantity().intValue());
            return itemParam;
        }).collect(Collectors.toList());
        takeNumParam.setItems(itemParamList);

        List<WaybillCloudPrintResponse> waybillCloudPrintResponses = takeNum(takeNumParam);

        orderEncasement.setTrackingNumber(waybillCloudPrintResponses.get(0).getWaybillCode());
        orderEncasementService.updateById(orderEncasement);
    }

    public List<WaybillCloudPrintResponse> takeNum(TakeNumParamDto dto) {
        PacClient client = new PacClient(basicKeyBo.getAppKey(), basicKeyBo.getSecretKey(), basicKeyBo.getUrl());

        SendSysParams params = new SendSysParams();
        params.setFromCode(basicKeyBo.getToken());

        TmsWaybillGetRequest request = new TmsWaybillGetRequest();
        // 配送公司编码
        request.setCpCode(dto.getCpCode());
        // 发件人信息
        request.setSender(getSender(dto));
        // 交易订单信息
        request.setTradeOrderInfoDtos(getTradeOrderInfoDtoList(dto));

        TmsWaybillGetResponse response = client.send(request, params);

        if (response.isSuccess()) {
            return response.getWaybillCloudPrintResponseList();
        }

        throw new ServiceException(response.getErrorMsg());
    }

    /**
     * 获取交易订单信息
     */
    private List<TradeOrderInfoDto> getTradeOrderInfoDtoList(TakeNumParamDto dto) {
        List<TradeOrderInfoDto> tradeOrderInfoDtoList = new ArrayList<>();

        // 收件信息
        TradeOrderInfoDto tradeOrderInfoDto = new TradeOrderInfoDto();
        tradeOrderInfoDto.setObjectId(dto.getObjectId());
        tradeOrderInfoDto.setTemplateUrl(dto.getEmsTemplateUrl());
        tradeOrderInfoDto.setOrderInfo(getOrderInfoDto(dto));
        tradeOrderInfoDto.setPackageInfo(getPackageInfo(dto));
        tradeOrderInfoDto.setRecipient(getRecipient(dto));
        tradeOrderInfoDtoList.add(tradeOrderInfoDto);

        return tradeOrderInfoDtoList;
    }

    /**
     * 订单信息
     */
    private OrderInfoDto getOrderInfoDto(TakeNumParamDto dto) {
        OrderInfoDto orderInfoDto = new OrderInfoDto();
        orderInfoDto.setOrderChannelsType("OTHERS");
        orderInfoDto.setTradeOrderList(Collections.singletonList(dto.getOrderNum()));
        return orderInfoDto;
    }

    /**
     * 包装信息
     */
    private PackageInfoDto getPackageInfo(TakeNumParamDto dto) {
        // 商品信息
        List<Item> itemList = dto.getItems().stream().map(itemParamDto -> {
            Item item = new Item();
            item.setName(itemParamDto.getItemName());
            item.setCount(itemParamDto.getItemCount());
            return item;
        }).collect(Collectors.toList());

        PackageInfoDto packageInfoDto = new PackageInfoDto();
        packageInfoDto.setWeight(dto.getWeight());
        packageInfoDto.setVolume(dto.getVolume());
        packageInfoDto.setItems(itemList);
        return packageInfoDto;
    }

    /**
     * 获取发件人发件地址
     */
    private UserInfoDto getSender(TakeNumParamDto dto) {
        // 寄件地址，这里的地址需要是卖家订购电子面单服务时使用的订购地址，具体可以通过TMS_WAYBILL_SUBSCRIPTION_QUERY接口获取
        AddressDto sendAddress = new AddressDto();
        sendAddress.setProvince(dto.getProvince());
        sendAddress.setCity(dto.getCity());
        sendAddress.setDistrict(dto.getDistrict());
        sendAddress.setDetail(dto.getDetail());

        UserInfoDto sender = new UserInfoDto();
        sender.setName(dto.getName());
        sender.setPhone(dto.getPhone());
        sender.setMobile(dto.getMobile());
        sender.setAddress(sendAddress);
        return sender;
    }

    /**
     * 获取收件人收件地址
     */
    private UserInfoDto getRecipient(TakeNumParamDto dto) {
        AddressDto receiveAddress = new AddressDto();
        receiveAddress.setProvince(dto.getReceiveProvince());
        receiveAddress.setCity(dto.getReceiveCity());
        receiveAddress.setDistrict(dto.getReceiveDistrict());
        receiveAddress.setDetail(dto.getReceiveDetail());

        UserInfoDto receiver = new UserInfoDto();
        receiver.setName(dto.getReceiveName());
        receiver.setMobile(dto.getReceiveMobile());
        receiver.setAddress(receiveAddress);
        return receiver;
    }

}
